# Modélisation de la caractéristique d'une cellule photovoltaïque

import matplotlib.pyplot as plt
import numpy as np

#paramètres de base. Attention, éviter de trop les modifier sous peine d'incohérences
#Umpp doit rester inférieur à Uoc
Pmpp=3.2#Puissance maximale
Icc=0.6#proportionnel à l'éclairement
Uoc=9.5
Umpp=6.8#tension de puissance maximale
Impp=Pmpp/Umpp#Intensité de puissance maximale

I=np.arange(0, 1, 0.001)
#d'après un modèle trouvé dans la littérature :
B=(Impp/(Icc-Impp)+np.log((Icc-Impp)/Icc))/(2*Umpp/Uoc-1)
Rs=Pmpp/Impp**2-Uoc/B*(1/(Icc-Impp))

couleur1='royalblue'
couleur2='orangered'
fig, ax1=plt.subplots()
plt.title("Caractéristique d'une cellule photovoltaïque")

U=Uoc*(1+1/B*np.log((Icc-I)/Icc))-Rs*I#modèle
P=U*I

ax1.set_xlabel('tension (V)')

#Axe ordonnées intensité :

ax1.scatter(Umpp, Impp, color='green')#Affichage du point de puissance maximale

ax1.set_ylabel('intensité (A)', color=couleur1)
ax1.plot(U, I, color=couleur1)
ax1.tick_params(axis='y',labelcolor=couleur1)
ax1.set_ylim(0, Icc*1.1)

#Axe ordonnées puissance :
ax2 = ax1.twinx()  # 2e axe des ordonnées partageant la même abscisse
ax2.scatter(Umpp, Pmpp, color='green')#Affichage du point de puissance maximale

ax2.set_ylabel('Puissance (W)', color=couleur2)
ax2.plot(U, P, color=couleur2)
ax2.tick_params(axis='y',labelcolor=couleur2)
ax1.set_xlim(0, Uoc*1.05)
ax2.set_ylim(0, max(P)*1.1)
#fig.tight_layout()  # otherwise the right y-label is slightly clipped
plt.show()
